<?php
/**
 * Ágora 2.3.4
 *
 * Un marco de desarrollo web sencillo.

  Copyright (c) 2009-2023 Guillermo Martínez J.

  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
    claim that you wrote the original software. If you use this software
    in a product, an acknowledgment in the product documentation would be
    appreciated but is not required.

    2. Altered source versions must be plainly marked as such, and must not be
    misrepresented as being the original software.

    3. This notice may not be removed or altered from any source
    distribution.

 */

/**
 * Identifica la versión de Ágora.
 */
  define ('AG_VERSION', '2.3.4');

/**
 * Identifica la versión mayor de Ágora.
 */
  define ('AG_V_MAYOR', 2);

/**
 * Identifica la rama menor de la versión.
 */
  define ('AG_V_MENOR', 3);

/**
 * Identifica el número de correcciones de la versión.
 */
  define ('AG_V_CORRECCIONES', 4);

/**
 * Directorio donde se encuentra ágora.
 */
  if (!defined ('AG_DIR'))
    define ('AG_DIR', __DIR__ . DIRECTORY_SEPARATOR);

/**
 * Directorio de vistas.
 */
  if (!defined ('AG_DIR_VISTAS'))
    define ('AG_DIR_VISTAS', AG_DIR . 'vistas' . DIRECTORY_SEPARATOR);

/**
 * Nombre de la vista para errores 404.
 */
  if (!defined ('AG_VISTA_404'))
    define ('AG_VISTA_404', 'pag_error404');

/**
 * Nombre de la vista para errores genéricos.
 */
  if (!defined ('AG_VISTA_ERROR'))
    define ('AG_VISTA_ERROR', 'pag_error');

/**
 * Directorio de módulos.  Esta constante es utilizada por AGORA::CargaModulo.
 */
  if (!defined ('AG_DIR_MODULOS'))
    define ('AG_DIR_MODULOS', AG_DIR . 'modulos' . DIRECTORY_SEPARATOR);



/**
 * Define valor para redirección temporal.
 */
  define ('REDIR_TMP', 307);
/**
 * Define valor para redirección permanente.
 */
  define ('REDIR_PERMANENTE', 301);



/**
 * Clase contenedora de Ágora.
 *
 * Puede usarse como base para otras clases, creando un objeto o llamando
 * directamente a los métodos estáticos.
 */
  class AGORA
  {
  /**
   * Devuelve el valor de la variable de configuración, si existe, o el valor
   * $Defecto si no existe.
   *
   * La configuración se encuentra en un diccionario global llamado $Config.
   * Este diccionario no es creado por Ágora.
   */
    public static function Config ($NombreVar, $Defecto = FALSE)
    {
      global $Config;

      return isset ($Config[$NombreVar]) ? $Config[$NombreVar] : $Defecto;
    }



  /**
   * Cambia los caracteres '&', '%', '<' y '>' para evitar inyecciones.
   * Es posible que no evite absolutamente todas las inyecciones, pero he hecho
   * algunas pruebas y funciona razonablemente bien.
   *
   * Otra utilidad que tiene es que si se le pasa código fuente (incluso XML),
   * lo devuelve en un formato que permite mostrarlo sin ningún problema.
   */
    public static function FiltroXSS ($Texto)
    {
    # Si es una lista, procésala una a una.
      if (is_array ($Texto)):
	foreach ($Texto as $Ndx => $Item)
	  $Texto[$Ndx] = self::FiltroXSS ($Item);
	return $Texto;
      endif;
    # Realiza el filtrado.
      if (is_string ($Texto))
	return str_replace (
	  array ('&', '%', '<', '>'),
	  array ('&amp;', '&#37;', '&lt;', '&gt;'),
	  $Texto
	);
      else
	return $Texto;
    }



  /**
   * Función de ayuda para obtener elementos de una lista.
   */
    public static function ElementoLista (array $Lista, $Nombre, $FiltroXSS=FALSE)
    {
    # Si no solicita un nombre, devuelve la lista completa.
      if ($Nombre === NULL and !empty ($Lista))
	return ($FiltroXSS === TRUE)
	     ? self::FiltroXSS ($Lista)
	     : $Lista;
    # En otro caso, devuelve el elemento solicitado.
      if (isset ($Lista[$Nombre]))
	return ($FiltroXSS === TRUE)
	     ? self::FiltroXSS ($Lista[$Nombre])
	     : $Lista[$Nombre];
    # No existe.
      return FALSE;
    }



  /**
   * Devuelve el valor del parámetro GET solicitado o .F. si este no se
   * recibió.
   */
    public static function get ($Nombre = NULL, $FiltroXSS = FALSE)
    {
      return self::ElementoLista ($_GET, $Nombre, $FiltroXSS);
    }



  /**
   * Devuelve el valor del parámetro POST solicitado o .F. si este no se
   * recibió.
   */
    public static function post ($Nombre = NULL, $FiltroXSS = FALSE)
    {
      return self::ElementoLista ($_POST, $Nombre, $FiltroXSS);
    }



  /**
   * Devuelve el valor del parámetro POST o GET solicitado o .F. si este no se
   * recibió.
   *
   * Si está en ambas entradas, POST tiene preferencia.
   */
    public static function get_post ($Nombre, $FiltroXSS = FALSE)
    {
      if (isset ($_POST[$Nombre]))
	return self::ElementoLista ($_POST, $Nombre, $FiltroXSS);
      return self::ElementoLista ( $_GET, $Nombre, $FiltroXSS);
    }



    private static $_CfgHuellas = array (
      'ruta' => '', 'dominio' => '', 'https' => FALSE, 'javascript' => FALSE
    );




  /**
   * Modifica la configuración del sistema de huellas (/cookies/).
   * Puede recibir el nombre del parámetro y el valor, donde parámetro puede
   * ser:
   * * 'ruta':  Indica la ruta (/path/) para las huellas.
   * * 'dominio': Indica el dominio para las huellas.
   * * 'https': Indica si debe usarse el protocolo HTTPS.
   * * 'javascript': Indica si se podrá o no acceder a los valores a través de
   *                 JavaScript.
   * También puede recibir un diccionario (/array/) en el que el índice es el
   * nombre del parámetro.
   *
   * Si el parámetro indicado no existe, símplemente se ignora su valor.
   */
    public static function ConfigHuellas ($Parametro, $Valor = NULL)
    {
      if (is_array ($Parametro)):
	foreach ($Parametro as $Ndx => $Item)
	  self::ConfigHuellas ($Ndx, $Item);
      else:
	if (isset (self::$_CfgHuellas[$Parametro]))
	  self::$_CfgHuellas[$Parametro] = $Valor;
      endif;
    }



  /**
   * Devuelve la configuración de las huellas en forma de diccionario.
   */
    public static function TomaConfigHuellas ()
    {
      return self::$_CfgHuellas;
    }



  /**
   * Ayuda para poner huellas.
   */
    private static function EstableceValoresHuella ($Nombre, $Valor, $Caducidad)
    {
      return setcookie ($Nombre, $Valor, $Caducidad,
	self::$_CfgHuellas['ruta'],
	self::$_CfgHuellas['dominio'],
	self::$_CfgHuellas['https'],
	!self::$_CfgHuellas['javascript']
      );
    }



  /**
   * Establece una huella (/cookie/).
   * @param string $Nombre Nombre de la variable.
   * @param        $Valor Valor a asignar a la variable.
   * @param int    $Caducidad Fecha de caducidad.  Es una fecha Unix.  En otras
   *                          palabras, probablemente utilizará la función
   *                          time() más el número de segundos que quiere que
   *                          dure. Si se pone 0, la huella expirará al final de
   *                          la sesión (al cerrarse el navegador).
   */
    public static function PonHuella ($Nombre, $Valor, $Caducidad)
    {
      if ($Caducidad !== 0 and $Caducidad <= time ()):
	self::EliminaHuella ($Nombre);
	return;
      endif;
      if (!self::EstableceValoresHuella ($Nombre, $Valor, $Caducidad))
	throw new Exception ("No pudo establecerse la huella '$Nombre'.");
    }



  /**
   * Devuelve el contenido de una huella o .F. si esta no se encontró.
   */
    public static function TomaHuella ($Nombre, $FiltroXSS = FALSE)
    {
      return self::ElementoLista ($_COOKIE, $Nombre, $FiltroXSS);
    }



  /**
   * Elimina la huella indicada, de existir.
   */
    public static function EliminaHuella ($Nombre)
    {
    # Si es una lista, elimínalas todas.
      if (is_array ($Nombre)):
	foreach ($Nombre as $Huella)
	  self::EstableceValoresHuella ($Huella, FALSE, time () - 3600);
      else:
	self::EstableceValoresHuella ($Nombre, FALSE, time () - 3600);
      endif;
    }



  /**
   * Elimina todas las huellas disponibles.
   */
    public static function LimpiaHuellas ()
    {
      self::EliminaHuella (array_keys ($_COOKIE));
    }



  /**
   * Carga y procesa la vista.
   *
   * @param string $Vista Nombre de la vista.
   * @param array $Vars Diccionario con las variables usadas por la vista.
   * @param bool $Devolver Devolver la vista procesada como cadena (TRUE) o enviar el resultado al cliente (FALSE).
   */
    public static function CargaVista ($_Vista, $_Vars = array (), $_Devolver = FALSE)
    {
      $_NombreArchivo =  AG_DIR_VISTAS . "$_Vista.php";
      if (!is_file ($_NombreArchivo))
	throw new Exception ("No se encontró la plantilla '$_Vista'", 404);
    # Si recibes una lista de parámetros, genera las variables precisas.
    # Nótese que no sobreescribe las ya existentes para impedir la modificación
    # de las variables globales.
      if (is_array ($_Vars) and count ($_Vars) > 0)
	extract ($_Vars, EXTR_SKIP);
    # Genera la vista.
      ob_start ();
      if ((bool) @ini_get ('short_open_tag') === FALSE):
      # El intérprete no permite etiqueta de apertura corta ("<?="), así que
      # sustitúyelo por "<?php echo ", que es equivalente.  Esto lo he tomado
      # prestado de CodeIgniter 2.
	echo eval (
	  '?'.'>' . preg_replace (
	    '/;*\s*\?'.'>/',
	    '; ?'.'>',
	    str_replace (
	      '<'.'?=',
	      '<'.'?php echo ',
	      file_get_contents ($_NombreArchivo)
	    )
	  )
	);
      else:
	require ($_NombreArchivo);
      endif;
    # Comprueba si debes devolver el resultado.
      if ($_Devolver):
	$_Resultado = ob_get_contents ();
	ob_end_clean ();
	return $_Resultado;
      endif;
      ob_end_flush ();
    }



  /**
   * Contenedor de objetos de módulo.  Es asignado por AGORA::CargaModulo.
   */
    static $Modulos = NULL;



  /**
   * Carga la definición de clase del archivo indicado y crea un objeto
   * asignándolo como meta-propiedad de AGORA::$Modulos.
   * Recibe el nombre del módulo y, opcionalmente, el nombre del objeto.
   * Observe que el nombre del módulo ha de estar en minúsculas, mientras que
   * el de la clase debe estar en mayúsculas, así, si el módulo se llama
   * "modulo", el archivo se llamará "modulo.php" y la clase "MODULO".
   *
   * Si se carga el módulo varias veces con el mismo nombre de objeto, ignora
   * las cargas adicionales.
   *
   * Si como nombre de objeto se pasa un valor "FALSE" entonces no crea ningún
   * objeto.
   */
    public static function &CargaModulo ($NombreModulo, $NombreObjeto = '')
    {
      if (!isset (self::$Modulos)) self::$Modulos = new TListamodulos ();
      $IdModulo = strtolower ($NombreModulo);
    # Comprueba si cargaste ya el módulo y si necesitas el nombre del objeto.
      if ($NombreObjeto === ''
      or !isset (self::$Modulos->_Archivos[$IdModulo])):
      # Si el módulo está en un subdirectorio, extrae la ruta y sepáralo del
      # nombre de módulo.
	if (strpos ($NombreModulo, '/') === FALSE):
	  $RutaModulo = '';
	else:
	  $ListaDir = explode ('/', $NombreModulo);
	  $NombreModulo = end ($ListaDir);
	  unset ($ListaDir[count ($ListaDir) - 1]);
	  $RutaModulo = implode ('/', $ListaDir).'/';
	endif;
      # Retoca los nombres.
	if ($NombreObjeto === '') $NombreObjeto = $NombreModulo;
	$NombreClase = strtoupper ($NombreModulo);
	$NombreModulo = strtolower ($NombreModulo);
      # Carga el módulo, si es preciso.
	if (!isset (self::$Modulos->_Archivos[$IdModulo])):
	  $NombreArchivo =  AG_DIR_MODULOS . "$RutaModulo$NombreModulo.php";
	  if (!is_file ($NombreArchivo))
	    throw new Exception (
	      'El módulo "' . basename ($NombreArchivo) . '" no existe',
	      404
	    );
	  include_once $NombreArchivo;
	  self::$Modulos->_Archivos[$IdModulo] = $NombreClase;
	endif;
      else:
	$NombreClase = self::$Modulos->_Archivos[$IdModulo];
      endif;
    # Ahora encárgate de si debes crear el objeto, y si puedes hacerlo.
      if ($NombreObjeto !== FALSE and !isset (self::$Modulos->$NombreObjeto)):
	if (class_exists ($NombreClase))
	  self::$Modulos->$NombreObjeto = new $NombreClase ();
	else
	  throw new Exception (
	    "No se encuentra la clase \"$NombreClase\" del módulo \"$IdModulo\".",
	    500
	  );
      endif;
    # Finalmente, devuelve el valor solicitado.
      if (isset (self::$Modulos->$NombreObjeto))
	return self::$Modulos->$NombreObjeto;
      else
	return $NombreObjeto;
    }



  /**
   * Permite acceder a los módulos de forma más simple si se ha creado un
   * objeto con la clase AGORA.  Así, puede accederse a
   * $MiObjeto->Modulos->Objeto como $MiObjeto->Objeto.
   */
    function __get ($key)
    {
      return self::$Modulos->$key;
    }



  /**
   * Redirige al cliente a otra URL.
   */
    public static function Redirige ($URL, $Razon = REDIR_TMP)
    {
    # TODO: Comprobar que la URL sea válida.
      self::EnviaCabeceraHTTP ($Razon);
      header ("Location: $URL");
    }



  /**
   * Lista de códigos de error HTTP.
   */
    public static $ErrorHTTP = array (
	200 => 'OK',
	201 => 'Created',
	202 => 'Accepted',
	203 => 'Non-Authoritative Information',
	204 => 'No Content',
	205 => 'Reset Content',
	206 => 'Partial Content',

	300 => 'Multiple Choices',
	301 => 'Moved Permanently',
	302 => 'Found',
	304 => 'Not Modified',
	305 => 'Use Proxy',
	307 => 'Temporary Redirect',

	400 => 'Bad Request',
	401 => 'Unauthorized',
	403 => 'Forbidden',
	404 => 'Not Found',
	405 => 'Method Not Allowed',
	406 => 'Not Acceptable',
	407 => 'Proxy Authentication Required',
	408 => 'Request Timeout',
	409 => 'Conflict',
	410 => 'Gone',
	411 => 'Length Required',
	412 => 'Precondition Failed',
	413 => 'Request Entity Too Large',
	414 => 'Request-URI Too Long',
	415 => 'Unsupported Media Type',
	416 => 'Requested Range Not Satisfiable',
	417 => 'Expectation Failed',

	500 => 'Internal Server Error',
	501 => 'Not Implemented',
	502 => 'Bad Gateway',
	503 => 'Service Unavailable',
	504 => 'Gateway Timeout',
	505 => 'HTTP Version Not Supported'
      );



  /**
   * Devuelve al cliente una cabecera HTTP determinada.
   * Devuelve a la función llamante el "Texto" definitivo.
   */
    public static function EnviaCabeceraHTTP ($Codigo = 500, $Texto = '')
    {
    # Comprueba que los códigos sean correctos.
      if (!is_integer ($Codigo) or $Codigo <= 0) $Codigo = 500;
    # Obtén el mensaje de cabecera, según su código y asignación.
      if ($Texto == ''):
	if (isset (self::$ErrorHTTP[$Codigo])):
	  $Texto = self::$ErrorHTTP[$Codigo];
	else:
	  self::Error (
	    "No se encontró cabecera para el error '$Codigo'.&nbsp; Por favor, use el código adecuado o provea su propia cabecera."
	  );
	endif;
      endif;
    # Envía la cabecera.
      $Protocolo = (isset ($_SERVER['SERVER_PROTOCOL']))
		 ? $_SERVER['SERVER_PROTOCOL']
		 : FALSE;
      if (substr (php_sapi_name (), 0, 3) == 'cgi')
	header ("Status: $Codigo $Texto", TRUE);
      elseif ($Protocolo == 'HTTP/1.1' or $Protocolo == 'HTTP/1.0')
	header ("$Protocolo $Codigo $Texto", TRUE, $Codigo);
      else
	header ("HTTP/1.1 $Codigo $Texto", TRUE, $Codigo);
      return $Texto;
    }



  /**
   * Devuelve al cliente una página de error y termina.
   *
   * La página de error es una vista llamada "pag_error.php", y usa las
   * variables siguientes:
   * * codigo: Número del código de error.
   * * nombre: Texto que define el error HTTP.
   * * mensaje: Mensaje de error.
   *
   * Este método también envía el código de error HTTP en la cabecera.
   */
    public static function Error ($Mensaje, $Codigo = 500, $Nombre = '')
    {
    # Envía la cabecera.
      if (!is_integer ($Codigo) or $Codigo <= 0) $Codigo = 500;
      $Nombre = self::EnviaCabeceraHTTP ($Codigo, $Nombre);
    # Envía la página de error.
      if (is_array ($Mensaje)) $Mensaje = implode ('</p><p>', $Mensaje);
      self::CargaVista (AG_VISTA_ERROR, array (
	'nombre' => $Nombre,
	'codigo' => $Codigo,
	'mensaje' => "<p>$Mensaje</p>"
      ));
    # Termina la ejecución.
      exit;
    }



  /**
   * Igual que "Error", solo que devuelve el código de error 404 y usa otra
   * plantilla, llamada "pag_error404.php", que usa las variables "URL" y
   * "Mensaje".
   */
    public static function Error404 ($URL, $Mensaje = 'Página no encontrada.')
    {
    # Envía la cabecera.
      self::EnviaCabeceraHTTP (404);
    # Envía la página de error.
      if (is_array ($Mensaje)) $Mensaje = implode ('</p><p>', $Mensaje);
      self::CargaVista (AG_VISTA_404, array (
	'URL' => $URL,
	'mensaje' => "<p>$Mensaje</p>"
      ));
    # Termina la ejecución.
      exit;
    }
  };



/* Clase contenedora de los módulos.
 */
  CLASS TListamodulos { public $_Archivos = array (); };
